wx-login(){
    if [[ -z "$HOSTNAME" || ${#HOSTNAME} -lt 5 ]]
    then
        wx-header "Login"
        echo "Status: Hostname Required"
        wx-stop
    fi

    if [[ ! -z $1 ]]
    then
        ORG=$1
    elif [[ $(hostname -d) = *"devices.waren.io" ]]
    then
        ORG=warengroup
    elif [[ $(hostname -d) = *"devices.cwinfo.net" ]]
    then
        ORG=cwinfo
    elif [[ $(hostname -d) = *"devices.christerwaren.fi" ]]
    then
        ORG=cwchristerw
    else
        wx-header "Login"
        echo "Status: Organization Required"
        wx-stop
    fi

    if [[ $ORG == "warengroup" ]]
    then
        DOMAIN=waren.io
    elif [[ $ORG == "cwinfo" ]]
    then
        DOMAIN=cwinfo.net
    elif [[ $ORG == "cwchristerw" ]]
    then
        DOMAIN=christerwaren.fi
    else
        wx-header "Login"
        echo "Status: Organization Unsupported"
        wx-stop
    fi

    FOLDER=$ORG
    DEVICE_DOMAIN="devices.$DOMAIN"
    IDM_DOMAIN="idm.cwinfo.net"
    VAULT_DOMAIN="vault.cwinfo.net"

    if [[ ! -z $2 ]]
    then
        HOSTNAME="$2.$DEVICE_DOMAIN"
    elif [[ $(hostname -d) ]]
    then
        HOSTNAME=$(hostname --fqdn)
    else
        wx-header "Login"
        echo "Status: Hostname Required"
        wx-stop
    fi

    if [[ -z $USER || $USER == "root" || $USER == "local" ]]
    then
        if [[ -z $SUDO_USER ]]
        then
            if [[ -z LOGNAME ]]
            then
                wx-header "Login"
                echo "Status: Username Required"
                wx-stop
            else
                USERNAME=$LOGNAME
            fi
        else
            USERNAME=$SUDO_USER
        fi
    else
        USERNAME=$USER
    fi

    VAULT_STATUS=$(curl -s -o /dev/null -w "%{http_code}" https://$VAULT_DOMAIN/v1/sys/health)
    if [[ $VAULT_STATUS -eq 200 ]]
    then

        if [[ $USER != "root" && $USER != "local" && -f "$HOME/.warengroup/config.json" ]]
        then
            TOKEN="$(cat $HOME/.warengroup/config.json | jq -r .login.$ORG)"
        fi

        VAULT_LOGIN=$(curl https://$VAULT_DOMAIN/v1/auth/token/renew -X POST --header "X-Vault-Token: $TOKEN" -d '{ "token": "'$TOKEN'" }' -s | jq -r '.auth.client_token')
        if [[ ! -z $VAULT_LOGIN && ${#VAULT_LOGIN} == 95 ]]
        then
            config["login",${ORG}]=$VAULT_LOGIN
            if [[ $USER != "root" && $USER != "local" ]]
            then
                jq '.login.'$ORG' = "'$VAULT_LOGIN'"' $HOME/.warengroup/config.json 1> $HOME/.warengroup/config.json.tmp
                mv $HOME/.warengroup/config.json.tmp $HOME/.warengroup/config.json &> /dev/null
            fi
        else
            IDM_STATUS=$(curl -s -o /dev/null -w "%{http_code}" https://$IDM_DOMAIN)
            if [[ $IDM_STATUS -eq 301 ]]
            then
                wx-header "Login"
                echo $wxBold$ORG$wxNormal

                if [[ -z $USERNAME || $USERNAME == "root" || $USERNAME == "local" ]]
                then
                    echo -n "Username: "
                    read USERNAME
                else
                    echo "Username: $USERNAME"
                fi

                echo -n "Password: "
                read -s PASSWORD
                echo "****************"

                if [[ -z $USERNAME || -z $PASSWORD ]]
                then
                    echo "Status: Username & Password Required"
                    wx-stop
                else
                    VAULT_LOGIN=$(curl https://$VAULT_DOMAIN/v1/auth/ldap/login/$USERNAME -X POST -d '{ "password": "'$PASSWORD'" }' -s | jq -r '.auth.client_token')
                    if [[ -z $VAULT_LOGIN || ${#VAULT_LOGIN} -lt 95 || ${#VAULT_LOGIN} -gt 95 ]]
                    then
                        echo "Status: Login Failed"
                        wx-stop
                    fi

                    config["login",${ORG}]=$VAULT_LOGIN

                    if [[ $USER != "root" && $USER != "local" ]]
                    then
                        jq '.login.'$ORG' = "'$VAULT_LOGIN'"' $HOME/.warengroup/config.json &> $HOME/.warengroup/config.json.tmp
                        mv $HOME/.warengroup/config.json.tmp $HOME/.warengroup/config.json &> /dev/null
                    fi
                fi
            else
                wx-header "Login"
                echo $wxBold$ORG$wxNormal

                echo -n "Token: "
                read -s TOKEN
                echo "****************"

                if [[ -z $TOKEN || ${#TOKEN} -lt 95 || ${#TOKEN} -gt 95 ]]
                then
                    echo "Status: Vault Token Required"
                    wx-stop
                fi

                VAULT_LOGIN=$(curl https://$VAULT_DOMAIN/v1/auth/token/renew -X POST --header "X-Vault-Token: $TOKEN" -d '{ "token": "'$TOKEN'" }' -s | jq -r '.auth.client_token')
                if [[ -z $VAULT_LOGIN || ${#VAULT_LOGIN} -lt 95 || ${#VAULT_LOGIN} -gt 95 ]]
                then
                    echo "Status: Login Failed"
                    wx-stop
                fi

                config["login",${ORG}]=$VAULT_LOGIN

                if [[ $USER != "root" && $USER != "local" ]]
                then
                    jq '.login.'$ORG' = "'$VAULT_LOGIN'"' $HOME/.warengroup/config.json &> $HOME/.warengroup/config.json.tmp
                    mv $HOME/.warengroup/config.json.tmp $HOME/.warengroup/config.json &> /dev/null
                fi
            fi
        fi
    else
        wx-header "Login"
        echo $wxBold$ORG$wxNormal
        echo "Status: Vault Offline"
        wx-stop
    fi
}
